package io.plumbr.avro;

import com.fasterxml.jackson.dataformat.avro.AvroMapper;
import com.fasterxml.jackson.dataformat.avro.AvroSchema;
import java.io.File;
import java.io.IOException;
import org.apache.avro.Schema;

public class Main {
  public static void main(String[] args) throws IOException {
    AvroMapper mapper = new AvroMapper();

    AvroSchema codeBasedSchema = mapper.schemaFor(MyClass.class);
    System.out.println("Current schema from source code:");
    System.out.println(codeBasedSchema.getAvroSchema().toString(true));

    write(mapper);
    read(mapper);
  }

  private static void write(AvroMapper mapper) throws IOException {
    MyClass aClass = new MyClass();
    aClass.setStringField("String value");
    aClass.setLongField(42);

    mapper.writeValue(new File("version1.bin"), aClass);
  }

  private static void read(AvroMapper avroMapper) throws IOException {
    AvroSchema schema = getSchema();

    avroMapper.readerFor(MyClass.class).with(schema).readValue(new File("version1.bin"));
  }

  private static AvroSchema getSchema() throws IOException {
    AvroSchema writerSchema = new AvroSchema(new Schema.Parser().parse(Main.class.getResourceAsStream("writer.avro")));
    AvroSchema readerSchema = new AvroSchema(new Schema.Parser().parse(Main.class.getResourceAsStream("reader.avro")));

    return writerSchema.withReaderSchema(readerSchema);
  }
}
